==> pvfs-1.5.3-pre1.tgz: pvfs-1.5.3-pre1/mgr/meta/md_stat.c <==
/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char md_stat_c[] = "$Header: /projects/cvsroot/pvfs/mgr/meta/md_stat.c,v 1.12 2001/05/25 19:54:46 rbross Exp $";
/* This file contains the fstat metadata file call for the PVFS	*/
/* Also assumes space for request has been allocated 					*/

#include "../../config.h"
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#ifdef HAVE_LINUX_TYPES_H
#include <linux/types.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <unistd.h>
#include <meta.h>
#include <req.h>
#include "metaio.h"

extern int errno;

int md_stat(char *name, mreq_p request, fmeta_p metar_p)
{
	fmeta meta1;
	int fd, length;
	dmeta dir;
	char temp[MAXPATHLEN];

	/* zero the metadata */
	memset(metar_p, 0, sizeof(fmeta));

	/* Open metadata file */	
	if ((fd = meta_open(name, O_RDONLY)) < 0) {
		if (errno != ENOENT) perror("md_stat, meta_open");
		return (-1);
	}

	/* Read metadata file - read is done before access to determine if its a
	 * directory or not (if its directory we need to do execute permission
	 * checking on the entire directory and not just the parent)  */	
	if (meta_read(fd, &meta1) < 0) {
		if (errno != EISDIR) {
			perror("md_stat: meta_read");
			meta_close(fd); /* try to close */
			return(-1);
		}
		else /* directory - just return the normal stats */ {
			/* check entire directory path for execute permissions */
			/* we know its a directory, so setting 0 here inplace of fd allows
			 * meta_access to forgo that check */
			if (meta_access(0, name, request->uid,
				request->gid, X_OK) < 0) {
				perror("md_stat: meta_access");
				meta_close(fd); /* try to close */
				return(-1);
			}
			/* get stats and fill in pvfs specific stuff */
			if (fstat(fd, &(metar_p->u_stat)) < 0) {
				perror("md_stat: fstat");
				meta_close(fd); /* try to close */
				return(-1);
			}
			/* Close metadata file */
			if (meta_close(fd) < 0) {
				perror("md_stat: meta_close");
				return(-1);
			}	
			if (get_dmeta(name, &dir) < 0) {
				perror("md_stat: get_dmeta");
				return(-1);
			}
			metar_p->u_stat.st_mode =
				S_IFDIR | ((S_IRWXO | S_IRWXG | S_IRWXU) & dir.dr_mode);

			metar_p->u_stat.st_uid = dir.dr_uid;
			metar_p->u_stat.st_gid = dir.dr_gid;
			return(0);
		}
	}

	/* ok, we know its a file */
	/* check execute permission on parent directory */
	strncpy(temp, name, MAXPATHLEN);
	length = get_parent(temp); /* strip off filename */
	/* if length<0, CWD is being used and directory permissions will
	 * not be checked pursuant to the UNIX method */
	if (length >= 0) {
		/* check for permissions to read from directory */
		if (meta_access(0, temp, request->uid, request->gid, X_OK) < 0) {
			perror("md_stat: meta_access");
			meta_close(fd); /* try to close */
			return(-1);
		}
	}

	/* Close metadata file */
	if (meta_close(fd) < 0) {
		perror("md_stat: meta_close");
		return(-1);
	}	

	*metar_p = meta1;
	metar_p->u_stat.st_mode =
	S_IFREG|((S_IRWXO|S_IRWXG|S_IRWXU) & metar_p->u_stat.st_mode);
	return (0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 */

==> pvfs-1.5.4.tgz: pvfs-1.5.4/mgr/meta/md_stat.c <==
/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/* This file contains the fstat metadata file call for the PVFS	*/
/* Also assumes space for request has been allocated 					*/

#include "../../config.h"
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#ifdef HAVE_LINUX_TYPES_H
#include <linux/types.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <unistd.h>
#include <meta.h>
#include <req.h>
#include "metaio.h"

extern int errno;

int md_stat(char *name, mreq_p request, fmeta_p metar_p)
{
	fmeta meta1;
	int fd, length;
	dmeta dir;
	char temp[MAXPATHLEN];

	/* zero the metadata */
	memset(metar_p, 0, sizeof(fmeta));

	/* Open metadata file */	
	if ((fd = meta_open(name, O_RDONLY)) < 0) {
		if (errno != ENOENT) perror("md_stat, meta_open");
		return (-1);
	}

	/* Read metadata file - read is done before access to determine if its a
	 * directory or not (if its directory we need to do execute permission
	 * checking on the entire directory and not just the parent)  */	
	if (meta_read(fd, &meta1) < 0) {
		if (errno != EISDIR) {
			perror("md_stat: meta_read");
			meta_close(fd); /* try to close */
			return(-1);
		}
		else /* directory - just return the normal stats */ {
			/* check entire directory path for execute permissions */
			/* we know its a directory, so setting 0 here inplace of fd allows
			 * meta_access to forgo that check */
			if (meta_access(0, name, request->uid,
				request->gid, X_OK) < 0) {
				perror("md_stat: meta_access");
				meta_close(fd); /* try to close */
				return(-1);
			}
			/* get stats and fill in pvfs specific stuff */
			{
				struct stat sb;
				if (fstat(fd, &sb) < 0) {
					perror("md_stat: fstat");
					meta_close(fd); /* try to close */
					return(-1);
				}
				COPY_STAT(&metar_p->u_stat, &sb);
			}
			/* Close metadata file */
			if (meta_close(fd) < 0) {
				perror("md_stat: meta_close");
				return(-1);
			}	
			if (get_dmeta(name, &dir) < 0) {
				perror("md_stat: get_dmeta");
				return(-1);
			}
			metar_p->u_stat.st_mode =
				S_IFDIR | ((S_IRWXO | S_IRWXG | S_IRWXU) & dir.dr_mode);

			metar_p->u_stat.st_uid = dir.dr_uid;
			metar_p->u_stat.st_gid = dir.dr_gid;
			return(0);
		}
	}

	/* ok, we know its a file */
	/* check execute permission on parent directory */
	strncpy(temp, name, MAXPATHLEN);
	length = get_parent(temp); /* strip off filename */
	/* if length<0, CWD is being used and directory permissions will
	 * not be checked pursuant to the UNIX method */
	if (length >= 0) {
		/* check for permissions to read from directory */
		if (meta_access(0, temp, request->uid, request->gid, X_OK) < 0) {
			perror("md_stat: meta_access");
			meta_close(fd); /* try to close */
			return(-1);
		}
	}

	/* Close metadata file */
	if (meta_close(fd) < 0) {
		perror("md_stat: meta_close");
		return(-1);
	}	

	*metar_p = meta1;
	metar_p->u_stat.st_mode =
	S_IFREG|((S_IRWXO|S_IRWXG|S_IRWXU) & metar_p->u_stat.st_mode);
	return (0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 */
